﻿using Connected.Data;

namespace Connected.Globalization.Languages;

/// <summary>
/// Represents a language used by localization services in the environment.
/// </summary>
/// <remarks>
/// In a multilanguage enviroments this entity defines a set of languages
/// supported by the environment. Language contains LCID which is used when
/// choosing which localization strings are loaded in the request.
/// Language is typically set by user's identity thus enabling environment
/// globalization.
/// </remarks>
public interface ILanguage : IPrimaryKey<int>
{
	/// <summary>
	/// The name of the language. This is a descriptive property and does
	/// not have any special meaning when mapping globalization services.
	/// </summary>
	string Name { get; }
	/// <summary>
	/// The language status. If <see cref="Status.Disabled"/> the language is
	/// ignored even user has a language set to its identity.
	/// </summary>
	Status Status { get; }
	/// <summary>
	/// The locale id associated with a language. This value is used when a language
	/// is set as a request language and <see cref="System.Globalization.CultureInfo"/> is
	/// set during the request pipeline.
	/// </summary>
	int Lcid { get; }
	/// <summary>
	/// A set of mapping strings used when the environment is resolving a language from a browser.
	/// Browser sends supported languages and if identity is not set en evrionment tries to resolve
	/// a correct language via this property. Use ',' delimiter when setting multiple mappings.
	/// </summary>
	string Mappings { get; }
}
